@@ -11,7 +11,7 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""utilities for PyLint configuration :
+"""utilities for Pylint configuration :
 
 * pylintrc
 * pylint.d (PYLINTHOME)
